==> stringdiff_2.13-0.3.0-sources.jar: app/tulz/diff/TokenDiff.scala <==
package app.tulz.diff

import app.tulz.diff.format.DiffFormat
import app.tulz.diff.util.DiffLog.log
import app.tulz.diff.util.DiffCollapse
import app.tulz.diff.util.DiffPrettier
import app.tulz.diff.util.DiffTokenize

import scala.collection.mutable.ListBuffer

object TokenDiff {

  private val whitespace = "\\s+".r
  private def tokenize(s: String): IndexedSeq[String] = {
    val buffer = new ListBuffer[String]
    var pos    = 0
    whitespace.findAllMatchIn(s).foreach { m =>
      if (m.start > pos) {
        buffer.addOne(s.substring(pos, m.start))
      }
      buffer.addOne(s.substring(m.start, m.end))
      pos = m.end
    }
    if (pos < s.length) {
      buffer.addOne(s.substring(pos))
    }
    buffer.toIndexedSeq
  }

  def apply(
    actual: String,
    expected: String
  ): String = ansi(actual, expected)

  def ansi(
    actual: String,
    expected: String
  ): String =
    DiffFormat.ansi(diff(actual, expected))

  def ansiBoth(
    actual: String,
    expected: String
  ): (String, String) =
    DiffFormat.ansiBoth(diff(actual, expected))

  def text(
    actual: String,
    expected: String
  ): String =
    DiffFormat.ansi(diff(actual, expected))

  def raw(
    actual: String,
    expected: String
  ): List[DiffElement[String]] =
    diff(actual, expected)

  def diff[Out](
    s1: String,
    s2: String
  ): List[DiffElement[String]] = {
    val diff = SeqDiff(
      TokenDiff.tokenize(s1).view,
      TokenDiff.tokenize(s2).view,
      collapse = false
    )
    log(s"diff", diff)
    var in          = diff
    var transformed = diff
    var first       = true
    while (first || in != transformed) {
      first = false
      in = transformed

      transformed = DiffTokenize.moveWhitespace(transformed)
      log("whitespace", transformed)

      transformed = DiffTokenize.firstsGoFirst(transformed)
      log("inFirsts first", transformed)
    }
    transformed = DiffTokenize.join(transformed)

    transformed = DiffCollapse(transformed)

    DiffPrettier(transformed.map(_.map(_.mkString)))
  }

}

==> stringdiff_2.13-0.3.4-sources.jar: app/tulz/diff/TokenDiff.scala <==
package app.tulz.diff

import app.tulz.diff.format.DiffFormat
import app.tulz.diff.util.DiffCollapse
import app.tulz.diff.util.DiffPrettier
import app.tulz.diff.util.DiffTokenize
import compat._
import scala.collection.mutable.ListBuffer

object TokenDiff {

  private val whitespace = "\\s+".r
  private def tokenize(s: String): IndexedSeq[String] = {
    val buffer = new ListBuffer[String]
    var pos    = 0
    whitespace.findAllMatchIn(s).foreach { m =>
      if (m.start > pos) {
        buffer.append(s.substring(pos, m.start))
      }
      buffer.append(s.substring(m.start, m.end))
      pos = m.end
    }
    if (pos < s.length) {
      buffer.append(s.substring(pos))
    }
    buffer.toIndexedSeq
  }

  def apply(
    s1: String,
    s2: String
  ): String = ansi(s1, s2)

  def ansi(
    s1: String,
    s2: String
  ): String =
    DiffFormat.ansi(diff(s1, s2))

  def ansiBoth(
    s1: String,
    s2: String
  ): (String, String) =
    DiffFormat.ansiBoth(diff(s1, s2))

  def text(
    s1: String,
    s2: String
  ): String =
    DiffFormat.ansi(diff(s1, s2))

  def diff[Out](
    s1: String,
    s2: String
  ): List[DiffElement[String]] = {
    val diff = SeqDiff(
      TokenDiff.tokenize(s1).view,
      TokenDiff.tokenize(s2).view,
      collapse = false
    )
    var in          = diff
    var transformed = diff
    var first       = true
    while (first || in != transformed) {
      first = false
      in = transformed
      transformed = DiffTokenize.moveWhitespace(transformed)
      transformed = DiffTokenize.firstsGoFirst(transformed)
    }
    transformed = DiffTokenize.join(transformed)
    transformed = DiffCollapse(transformed)
    DiffPrettier(transformed.map(_.map(_.mkString)))
  }

}
